@@ -443,6 +443,12 @@ inline std::string qgetline(const QFile& qfile)
   if (size > 0) {
     std::string ret;
     const long pos = ftell(qfile.get_fp()) - qfile.p->offset_start;
+    if (pos < 0) {
+      qerr(
+          ssprintf("qgetline: '%s' initial_pos='%ld' final_pos='%ld' "
+                   "getline_size='%ld'.",
+                   qfile.path().c_str(), qfile.p->pos, pos, size));
+    }
     qassert(pos >= 0);
     if (qfile.p->offset_end != -1 and
         qfile.p->offset_start + pos > qfile.p->offset_end) {
@@ -926,7 +932,7 @@ inline bool read(const QarFile& qar, const std::string& fn, QFile& qfile_in)
   }
 }
 
-inline bool has(const QarFile& qar, const std::string& fn)
+inline bool has_regular_file(const QarFile& qar, const std::string& fn)
 // interface function
 {
   qassert(not qar.null());
@@ -937,11 +943,11 @@ inline bool has(const QarFile& qar, const std::string& fn)
   return read(qar, fn, qfile);
 }
 
-inline bool has_file_or_directory(const QarFile& qar, const std::string& fn)
+inline bool has(const QarFile& qar, const std::string& fn)
 // interface function
 {
   qassert(not qar.null());
-  if (has(qar, fn)) {
+  if (has_regular_file(qar, fn)) {
     return true;
   } else {
     qassert(qar.p->is_read_through);
