@@ -1,6 +1,6 @@
 #pragma once
 
-#include <qlat-utils/qutils.h>
+#include <qlat-utils/qutils-extra.h>
 #include <qlat-utils/rng-state.h>
 
 namespace qlat
@@ -146,139 +146,12 @@ qacc array<M, 10> make_array(const M& x, const M& x1, const M& x2, const M& x3,
   return arr;
 }
 
-template <class M>
-struct API Vector {
-  M* p;
-  long n;
-  //
-  qacc Vector<M>()
-  {
-    p = NULL;
-    n = 0;
-  }
-  qacc Vector<M>(const Vector<M>& vec)
-  {
-    p = vec.p;
-    n = vec.n;
-  }
-  template <int N>
-  qacc Vector<M>(const array<M, N>& arr)
-  {
-    p = (M*)arr.data();
-    n = arr.size();
-  }
-  Vector<M>(const std::vector<M>& vec)
-  {
-    p = (M*)vec.data();
-    n = vec.size();
-  }
-  qacc Vector<M>(const M* p_, const long n_)
-  {
-    p = (M*)p_;
-    n = n_;
-  }
-  qacc Vector<M>(const M& x)
-  {
-    p = (M*)&x;
-    n = 1;
-  }
-  //
-  qacc const M& operator[](const long i) const
-  {
-    if (not(0 <= i && i < n)) {
-#ifndef QLAT_IN_ACC
-      displayln(
-          ssprintf("ERROR: expect: 0 <= i && i < n but: i=%d n=%d sizeof(M)=%d",
-                   i, n, sizeof(M)));
-#endif
-      qassert(false);
-    }
-    return p[i];
-  }
-  qacc M& operator[](const long i)
-  {
-    if (not(0 <= i && i < n)) {
-#ifndef QLAT_IN_ACC
-      displayln(
-          ssprintf("ERROR: expect: 0 <= i && i < n but: i=%d n=%d sizeof(M)=%d",
-                   i, n, sizeof(M)));
-#endif
-      qassert(false);
-    }
-    return p[i];
-  }
-  //
-  qacc M* data() { return p; }
-  qacc const M* data() const { return p; }
-  //
-  qacc long size() const { return n; }
-  //
-  qacc long data_size() const { return n * sizeof(M); }
-  //
-  qacc const Vector<M>& operator=(const Vector<M>& v)
-  {
-    n = v.n;
-    p = v.p;
-    return *this;
-  }
-};
-
 template <class M>
 qacc void set_zero(Vector<M> vec)
 {
   std::memset((void*)vec.data(), 0, vec.data_size());
 }
 
-template <class M, int N>
-struct API Array {
-  M* p;
-  //
-  qacc Array<M, N>() { p = NULL; }
-  qacc Array<M, N>(const Array<M, N>& arr) { p = arr.p; }
-  qacc Array<M, N>(const Vector<M>& vec)
-  {
-    qassert(N == vec.size());
-    p = vec.p;
-  }
-  qacc Array<M, N>(const array<M, N>& arr) { p = (M*)arr.data(); }
-  qacc Array<M, N>(const M* p_) { p = (M*)p_; }
-  qacc Array<M, N>(const M& x)
-  {
-    qassert(N == 1);
-    p = (M*)&x;
-  }
-  //
-  qacc const M& operator[](int i) const
-  {
-    qassert(0 <= i && i < N);
-    return p[i];
-  }
-  qacc M& operator[](int i)
-  {
-    qassert(0 <= i && i < N);
-    return p[i];
-  }
-  //
-  qacc M* data() { return p; }
-  qacc const M* data() const { return p; }
-  //
-  qacc int size() const { return N; }
-  //
-  qacc long data_size() const { return N * sizeof(M); }
-  //
-  qacc const Array<M, N>& operator=(const Array<M, N>& v)
-  {
-    p = v.p;
-    return *this;
-  }
-  qacc const Array<M, N>& operator=(const Vector<M>& v)
-  {
-    qassert(N == v.size());
-    p = v.p;
-    return *this;
-  }
-};
-
 template <class M, int N>
 qacc void set_zero(Array<M, N> arr)
 {
@@ -764,4 +637,36 @@ std::vector<std::vector<M> > vector_split(const std::vector<M>& data,
   return datatable;
 }
 
+qacc bool qisnan(const float& arg) { return std::isnan(arg); }
+
+qacc bool qisnan(const double& arg) { return std::isnan(arg); }
+
+template <class T>
+bool qisnan(const std::complex<T>& arg)
+{
+  return qisnan(arg.real()) or qisnan(arg.imag());
+}
+
+template <class M, unsigned long N>
+qacc bool qisnan(const array<M, N>& arr)
+{
+  for (int i = 0; i < (int)N; ++i) {
+    if (qisnan(arr[i])) {
+      return true;
+    }
+  }
+  return false;
+}
+
+template <class M>
+bool qisnan(const std::vector<M>& arr)
+{
+  for (size_t i = 0; i < arr.size(); ++i) {
+    if (qisnan(arr[i])) {
+      return true;
+    }
+  }
+  return false;
+}
+
 }  // namespace qlat
